// =========================================================
// Copyright (c) Apache Project 2000.
// Time-stamp: <26 Nov 01 13:18:02 Jean-Guillaume.Dumas@imag.fr> 
// Modified from :

/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

//******************************************************************
// Function.java - A class containing a list of functions.  Each
//                  function can be integrated numerically or
//                  algebraicaly.  Used by the Integrate class.
//
// Author:  George Stebbins
//
//******************************************************************
 
package Integrate;


public abstract class Function {

    // The number of different functions

    public final static int COUNT = 12;


    // The array of functions

    static Function [] Functions = new Function[COUNT];
    

    // Methods of calulcation
    
    final static String [] methodNames      = { "Rectangular",
						"Trapezoidal",
						"Midpoint",
                                                                                       "Simpson",
                                                                                       "GaussLegendre",
						"None"};

    final static int       RECTANGULAR    = 0;  // rectangular
    final static int       TRAPEZOIDAL    = 1;  // trapezoidal
    final static int       MIDPOINT       = 2;  // midpoint
    final static int       SIMPSON        = 3;  // simpson
    final static int       GAUSSLEGENDRE        = 4;  // GaussLegendre
    final static int       NONE           = 5;  // No area calculated


    // Indexes for obtaining functions

    public final static int LINEAR = 0;
    public final static int SQUARED = 1;
    public final static int CUBED = 2;
    public final static int SQRT = 3;
    public final static int LN = 4;
    public final static int ETOX = 5;
    public final static int SIN = 6;
    public final static int COS = 7;
    public final static int TAN = 8;
    public final static int ONEOVERP = 9;
    public final static int ONEOVERN = 10;
    public final static int ONEOVERETOX = 11;
    

    // Attributes of each function

    public String name;       // Function name used for display

    public double lowX;       // Lower limit of integration
    public double highX;      // Upper limit of integration

    public double lowerX;     // Lower limit of x display
    public double higherX;    // Upper limit of x display

    public double lowerY;     // Lower limit of y display
    public double higherY;     // Upper limit of y display

    public double [] labels = {0};  // y-axis labels

    abstract public double integrand (double d);  // evaluates the function

    abstract public double integral (double d);   // evaluates the integral


    static {
	Functions[LINEAR] = new Linear();
	Functions[SQUARED] = new Squared();
	Functions[CUBED] = new Cubed();
	Functions[SQRT] = new Sqrt();
	Functions[LN] = new Ln();
	Functions[ETOX] = new Etox();
	Functions[SIN] = new Sin();
	Functions[COS] = new Cos();
	Functions[TAN] = new Tan();
	Functions[ONEOVERP] = new OneOverP();
	Functions[ONEOVERN] = new OneOverN();
	Functions[ONEOVERETOX] = new OneOverEtox();
    }

}




class Linear extends Function {

    public Linear () {
	name = "x";
	lowX = -10;
	highX = 10;
	lowerX = -11;
	higherX = 11;
	lowerY = -11;
	higherY = 11;
	labels = new double[3];
	labels[0] = 10;
	labels[1] = 0;
	labels[2] = -10;
    }

    public double integrand (double x) {
	return(x);
    }

    public double integral (double x) {
	return(x*x/2.0);
    }

}



class Squared extends Function {

    public Squared () {
	name = "x^2";
	lowX = -10;
	highX = 10;
	lowerX = -11;
	higherX = 11;
	lowerY = -10;
	higherY = 110;
	labels = new double[2];
	labels[0] = 100;
	labels[1] = 0;
    }

public double integrand (double x) {
	return(x*x);
    }

    public double integral (double x) {
	return(x*x*x/3.0);
    }

}


class Cubed extends Function {

    public Cubed () {
	name = "x^3";
	lowX = -5;
	highX = 5;
	lowerX = -5.5;
	higherX = 5.5;
	lowerY = -137.5;
	higherY = 137.5;
	labels = new double[3];
	labels[0] = -125;
	labels[1] = 0;
	labels[2] = 125;
    }

    public double integrand (double x) {
	return(x*x*x);
    }

    public double integral (double x) {
	return(x*x*x*x/4.0);
    }

}


class Sqrt extends Function {

    public Sqrt () {
	name = "sqrt(x)";
	lowX = 0;
	highX = 10;
	lowerX = -0.5;
	higherX = 10.5;
	lowerY = -0.3;
	higherY = 3.5;
	labels = new double[2];
	labels[0] = 0;
	labels[1] = 3;
    }

    public double integrand (double x) {
	double s;
	try {
	    if (x < 0)
		s = 0;
	    else
		s = Math.sqrt(x);
	} catch (ArithmeticException ae) {
	    s = 0;
	}
	return(s);
    }

    public double integral (double x) {
	return(2.0/3.0 * Math.pow(x, 3.0/2.0));
    }

}


class Ln extends Function {

    public Ln () {
	name = "ln(x)";
	lowX = 0.1;
	highX = 5.1;
	lowerX = -0.2;
	higherX = 5.4;
	lowerY = -2.5;
	higherY = 1.8;
	labels = new double[3];
	labels[0] = -2;
	labels[1] = 0;
	labels[2] = 1.5;
    }

    public double integrand (double x) {
	double s;
	try {
	    if (x <= 0)
		s = -10;
	    else
		s = Math.log(x);
	} catch (ArithmeticException ae) {
	    s = 0;
	}
	return(s);
    }

    public double integral (double x) {
	return(x*Math.log(x) - x);
    }

}



class Etox extends Function {

    public Etox () {
	name = "e^x";
	lowX = -3;
	highX = 3;
	lowerX = -3.3;
	higherX = 3.3;
	lowerY = -1.0;
	higherY = 22.0;
	labels = new double[2];
	labels[0] = 0;
	labels[1] = 20;
    }

    public double integrand (double x) {
	return(Math.pow(Math.E, x));
    }

    public double integral (double x) {
	return(Math.pow(Math.E, x));
    }

}



class Sin extends Function {

    public Sin () {
	name = "sin(x)";
	lowX = -7;
	highX = 7;
	lowerX = -7.7;
	higherX = 7.7;
	lowerY = -1.1;
	higherY = 1.1;
	labels = new double[3];
	labels[0] = 1;
	labels[1] = 0;
	labels[2] = -1;
    }

    public double integrand (double x) {
	return(Math.sin(x));
    }

    public double integral (double x) {
	return(-Math.cos(x));
    }

}


class Cos extends Function {

    public Cos () {
	name = "cos(x)";
	lowX = -7;
	highX = 7;
	lowerX = -7.7;
	higherX = 7.7;
	lowerY = -1.1;
	higherY = 1.1;
	labels = new double[3];
	labels[0] = 1;
	labels[1] = 0;
	labels[2] = -1;
    }

    public double integrand (double x) {
	return(Math.cos(x));
    }

    public double integral (double x) {
	return(Math.sin(x));
    }

}



class Tan extends Function {

    public Tan () {
	name = "tan(x)";
	lowX = -1.4;
	highX = 1.4;
	lowerX = -1.5;
	higherX = 1.5;
	lowerY = -7;
	higherY = 7;
	labels = new double[3];
	labels[0] = -5.5;
	labels[1] = 0;
	labels[2] = 5.5;
    }

    public double integrand (double x) {
	try {
	    return(Math.tan(x));
	} catch (ArithmeticException ae) {}
	return(100);
    }

    public double integral (double x) {
	return(-Math.log(Math.cos(x)));
    }

}


class OneOverP extends Function {

    public OneOverP () {
	name = "1/x (x>0)";
	lowX = 0.2;
	highX = 3.2;
	lowerX = -0.1;
	higherX = 3.5;
	lowerY = -0.5;
	higherY = 9;
	labels = new double[2];
	labels[0] = 0;
	labels[1] = 5;
    }

    public double integrand (double x) {
	if (x <= 0) return(100);
	try {
	    double s = 1/x;
	    if (s > 100) return(100);
	    return(s);
	} catch (ArithmeticException ae) {}
	return(100);
    }

    public double integral (double x) {
	return(Math.log(Math.abs(x)));
    }

}



class OneOverN extends Function {

    public OneOverN () {
	name = "1/x (x<0)";
	lowX = -3.2;
	highX = -0.2;
	lowerX = -3.5;
	higherX = 0.1;
	lowerY = -9;
	higherY = 0.5;
	labels = new double[2];
	labels[0] = 0;
	labels[1] = -5;
    }

    public double integrand (double x) {
	if (x >= 0) return(-100);
	try {
	    double s = 1/x;
	    if (s < -100) return(-100);
	    return(s);
	} catch (ArithmeticException ae) {}
	return(-100);
    }

    public double integral (double x) {
	return(Math.log(Math.abs(x)));
    }

}



class OneOverEtox extends Function {

    public OneOverEtox () {
	name = "1/(e^x)";
	lowX = -3;
	highX = 3;
	lowerX = -3.3;
	higherX = 3.3;
	lowerY = -1.5;
	higherY = 22.0;
	labels = new double[2];
	labels[0] = 0;
	labels[1] = 20;
    }

    public double integrand (double x) {
	return(1/Math.pow(Math.E, x));
    }

    public double integral (double x) {
	return(-1/Math.pow(Math.E, x));
    }

}

class Simpson {
    private double _x2, _x, _unit, _norm;

    public Simpson(double a, double b, double fa, double fm, double fb) {
        _x2 = 2*fb+2*fa-4*fm;
        _x = (4*fm-3*fb-fa)*a+(4*fm-fb-3*fa)*b;
        _unit = a*a*fb+(fb-4*fm+fa)*b*a+b*b*fa;
        _norm = (a-b)*(a-b);
    }
    
    public double integrand(double X) {
        return ((_x2*X*X+_x*X+_unit)/_norm);
    }
}

