// =========================================================
// Copyright (c) Apache Project 2000.
// Time-stamp: <08 Nov 00 19:33:29 Jean-Guillaume.Dumas@imag.fr> 
// Modified from :

/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

//******************************************************************
//  Integrate.java - Teaches numerical integration
//
//  Author:   George Stebbins
//  Modified: Joe Zachary
//
//******************************************************************

 
/*
<applet 
    CODEBASE=".."
    CODE="hamlet.AppButton.class" NAME="Integrate.Integrate"
    width=220 height=25 align=absbottom>
    <param name=LABEL value="Numerical Integration Simulation">
    A Java applet should appear here
</applet>
*/

package Integrate;

import java.awt.*;
import java.io.*;

import hamlet.AppFrame;
import hamlet.*;
import Integrate.*;


/* Allows experimentation with numerical integration techniques. */

public class Integrate extends AppFrame {

    ScrollableNumbers nRegions;  // Tracks number of regions in integration
    Plot thePlot;                // The integration plot

    int defaultRegions = 11;               // Initial number of regions
    int defaultFunction = Function.COS;   // Initial integrand
    int defaultMethod = Function.NONE;    // Initial integration method

    CheckboxMenu methodMenu;       // Menu of methods
    CheckboxMenu functionMenu;     // Menu of functions

    
    //******************************************************************
    // Constructor
    //******************************************************************
    
    public Integrate () {


	// Add method and function menus to menu bar
	
	methodMenu = new CheckboxMenu("Method");
	for (int i=0; i<Function.methodNames.length; i++)
	    methodMenu.add(Function.methodNames[i]);
	menuBar.add(methodMenu);
	
	functionMenu = new CheckboxMenu("Function");
	for (int i=0; i<Function.Functions.length; i++)
	    functionMenu.add(Function.Functions[i].name);
	menuBar.add(functionMenu);


	// Title and main layout.  There are two main regions: the
	// upper (center) region has the function plot, and the lower
	// (south) region has the controls.

	setTitle("Numerical Integration");
	setLayout (new BorderLayout(5,5));


	// Set up the lower (south) region
	
	Panel southPanel = new Panel();
	southPanel.setLayout(new BorderLayout());
	add ("South", southPanel);
	

	// This keeps track of the number of regions
	
	nRegions = new ScrollableNumbers(defaultRegions,
					 1,
					 10,
					 1,
					 1000,
					 4,
					 ScrollableNumbers.LEFT,
					 "Number of Evaluations ",
					 this);
	southPanel.add("North", nRegions);


	// This displays statistics

	IntegrationDisplay iDisplay = new IntegrationDisplay();
	southPanel.add("South", iDisplay);
	


	// The main function plot

	thePlot = new Plot(defaultRegions,
			   Function.Functions[defaultFunction],
			   defaultMethod,
			   iDisplay,
			   Color.gray,
			   Color.blue,
			   Color.white,
			   Color.red,
			   Color.green,
			   Color.black);
	add("Center", thePlot);
	

	// Set to defaults

	methodMenu.setChoice(defaultMethod);
        nRegions.setDoubleText(defaultRegions);
	functionMenu.setChoice(defaultFunction);

	helpText =
	    " This applet illustrates three methods for numerical\n" +
	    " integration as discussed in Chapter 15 of \"Introduction\n" +
	    " to Scientific Programming\" by Joseph L. Zachary.\n\n" +

	    " File Menu\n" +
	    "   + Reset:  Resets to initial appearance\n" +
	    "   + Hide:  Removes the applet from the display\n" +
	    "   + Quit:  Terminates the applet\n\n" +

	    " Method Menu\n" +
	    "   + Choose one of the methods for numerical integration\n\n" +

	    " Function Menu\n" +
	    "   + Choose a function to integrate\n\n" +
	    
	    " Help Menu\n" +
	    "   + With Applet:  Displays this help dialog\n\n" +

	    " Number of Regions\n" +
	    "   + Choose the number of regions to use when doing numerical\n" +
	    "     integration.  Must be between 1 and 1000.\n\n" +

	    " Mouse Actions\n" +
	    "   + Drag the upper and lower limits of integration lines\n" +
	    "     to change the limits of integration.\n\n" +

	    " Displayed Information\n" +
	    "   + The function being graphed and the integration method\n" +
	    "     in use (if any) is displayed above the plot.\n" +
	    "   + The bounds of integration are displayed below the plot.\n" +
	    "   + The actual value of the integral is displayed at the\n" +
	    "     bottom.\n" +
	    "   + If one of the three integration methods is being\n" +
	    "     displayed, the calculated value of the integral, the\n" +
	    "     difference from the actual value, and the percent\n" +
	    "     difference are all displayed.\n" +
	    "   + The percent difference is illustrated graphically with\n" +
	    "     a pie graph.  If the graph is all red, the percent\n" +
	    "     difference is at least 100%.  Otherwise, the amount of\n" +
	    "     red displayed is proportional to the percent difference.\n";

	pack();	

    }

    

    //******************************************************************
    // Reset everything to the default values
    //******************************************************************

    public void reset () {

	methodMenu.setChoice(defaultMethod);
        nRegions.setDoubleText(defaultRegions);
	functionMenu.setChoice(defaultFunction);

	thePlot.newMethod(defaultMethod);
 	thePlot.newFunction(Function.Functions[defaultFunction]);
	thePlot.newRegions(defaultRegions);

    }
    

    //**************************************************************
    // Action to do when a button is pushed
    //**************************************************************

    public boolean action (Event theEvent, Object argument) {


	// Give the superclass first crack.

	if (super.action(theEvent, argument)) return(true);
	

	// ScrollableNumbers means that the number of iterations has
	// changed
      
	if (theEvent.target instanceof ScrollableNumbers) {
	    thePlot.newRegions(nRegions.getIntText());
	    return true;

	}


	// The function or method may have changed

	else if (theEvent.target instanceof CheckboxMenuItem) {

	    if (methodMenu.update(theEvent.target)) {
		thePlot.newMethod(methodMenu.getChoice());
		return(true);
	    }

	    else if (functionMenu.update(theEvent.target)) {
		thePlot.newFunction(Function.Functions[functionMenu.getChoice()]);
		return(true);
	    }

	    else return(false);
	    
	}		                 
	    

	// Don't know what it is, so let higher levels handle it

	else {
	    return false;
	}
    }


}

