// =========================================================
// Copyright (c) Apache Project 2000.
// Time-stamp: <26 Nov 01 13:16:54 Jean-Guillaume.Dumas@imag.fr> 
// Modified from :

/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

//******************************************************************
// Plot.java - Contains a Plotting class.  (very simple)
//
// Author:  George Stebbins
//
//******************************************************************
 
package Integrate;

import java.io.*;

import java.awt.*;
import Integrate.*;


//******************************************************************
//******************************************************************
//
// Plot class 
//
//******************************************************************
//******************************************************************

class Plot extends Canvas {

    //----------------------------------------------------
    // Member data
    //----------------------------------------------------
    
    private int theWidth;
    private int theHeight;
    private int realWidth;
    private int realHeight;

    private double lowerLimit;
    private double upperLimit;

    private double sFactorX;
    private double sFactorY;

    private int numIterations;
    private Function theFunction;
    private int theMethod;
    
    private Color limitColor;
    private Color axesColor;
    private Color plotColor;
    private Color backColor;
    private Color boxColor;
    private Color labelColor;

    private double area;

    private final int tickSize = 5;
    private final int tickOffset = 2;
    private final double oneoversqrtthree = 0.5773502693;
    int border = 0;
    int leftBorder = 0;

    private boolean grabbedLower = false;
    private boolean grabbedUpper = false;

    private int grabX;
    int mouseX;

    private IntegrationDisplay idisplay;

    Image functionImage = null;

    
    //----------------------------------------------------
    // Constructor
    //----------------------------------------------------

    public Plot (int iterations, Function whichFunction,
		 int whichMethod, IntegrationDisplay idisplay,
		 Color backColor, Color axesColor,
		 Color limitColor, Color plotColor,
		 Color boxColor, Color labelColor) {
		     
       theFunction = whichFunction;		     
       resetLimits();
       theMethod = whichMethod;
       numIterations = iterations;
       this.idisplay = idisplay;

       this.backColor = backColor;
       this.axesColor = axesColor;
       this.limitColor = limitColor;
       this.plotColor = plotColor;
       this.boxColor = boxColor;
       this.labelColor = labelColor;

       idisplay.doShow(theMethod != Function.NONE);

    }
    
    
    //--------------------------------------------------
    // Handle resize and reshape requests
    //--------------------------------------------------

    public void resize (int w, int h) {
	adjustSize(w, h);
	super.resize(w, h);
    }


    public void resize (Dimension d) {
	adjustSize(d.width, d.height);
	super.resize(d);
    }

    public void reshape (int x, int y, int w, int h) {
	adjustSize(w, h);
	super.reshape(x,y,w,h);
    }

    public synchronized void adjustSize (int w, int h) {

	realHeight = h;
	realWidth = w;

	resetFunction(theFunction);

	sFactorX = (theWidth-1) / (theFunction.higherX-theFunction.lowerX);
	sFactorY = (theHeight-1) / (theFunction.higherY-theFunction.lowerY);
       
    }



    //--------------------------------------------------
    // When the mouse goes down, decide whether or not one of the
    // limit lines has been grabbed.
    //--------------------------------------------------

    public synchronized boolean mouseDown (Event evt, int x, int y) {

	// Remember the x-coordinate

	grabX = x;

	// Determine which divider the mouse is close to, if any.  Ties
	// go to the divider furthest from its limit.

	if (x > leftBorder+transX(lowerLimit)-3 &&
	        x < leftBorder+transX(lowerLimit)+3 &&
            x > leftBorder+transX(upperLimit)-3 &&
 	        x < leftBorder+transX(upperLimit)+3) {
	    if (lowerLimit - theFunction.lowX > theFunction.highX - upperLimit)
		grabbedLower = true;
	    else
		grabbedUpper = true;
	}
	        
	else if (x > leftBorder+transX(lowerLimit)-3 &&
	        x < leftBorder+transX(lowerLimit)+3) {
	    grabbedLower = true;
	}

	else if (x > leftBorder+transX(upperLimit)-3 &&
		     x < leftBorder+transX(upperLimit)+3) {
	    grabbedUpper = true;
	}

	return(true);

    }

 

    //--------------------------------------------------
    // If a limit line is being dragged, repaint.
    //--------------------------------------------------

    public synchronized boolean mouseDrag (Event evt, int x, int y) {

	if (grabbedLower || grabbedUpper) {
	    mouseX = x;
	    repaint();
	}
	return(true);

    }



    //--------------------------------------------------
    // If a limit line is being dragged, update and clean up.
    //--------------------------------------------------

    public synchronized boolean mouseUp (Event evt, int x, int y) {

	if (grabbedLower || grabbedUpper) {
	    grabbedLower = false;
	    grabbedUpper = false;
	    update(getGraphics());
	}
	return(true);
    }

    

    //--------------------------------------------------
    // Without this, the canvas is not initially visible.
    //--------------------------------------------------

    public Dimension preferredSize () {
	return(new Dimension(200,200));
    }


    //--------------------------------------------------
    // Change the number of regions
    //--------------------------------------------------

    public synchronized void newRegions (int iterations) {
	numIterations = iterations;
	paint(getGraphics());
    }


    //--------------------------------------------------
    // Change the function
    //--------------------------------------------------

    public synchronized void newFunction (Function whichFunction) {
	resetFunction(whichFunction);
	resetLimits();
	paint(getGraphics());
    }	


    //--------------------------------------------------
    // Change the method.  Hide part of the integration results
    // display if no numerical method is being used.
    //--------------------------------------------------

    public synchronized void newMethod (int whichMethod) {
	theMethod = whichMethod;
	idisplay.doShow(theMethod != Function.NONE);
	functionImage = null;
	paint(getGraphics());
    }	



    //--------------------------------------------------
    // Reset the various limiting parameters according to which
    // function is in use.
    //--------------------------------------------------

    private synchronized void resetFunction (Function whichFunction) {


	// Record the new function and require a new image

	theFunction = whichFunction;
	functionImage = null;

	
	// Set the left border width according to the size of this
	// function's labels, and the other border widths according
	// to the size of the font.

	FontMetrics fm = getFontMetrics(getFont());
	leftBorder = 0;
	for (int i = 0; i < theFunction.labels.length; ++i) {
	    String label = Double.toString(theFunction.labels[i]);
	    int size = fm.stringWidth(label);
	    if (size > leftBorder) leftBorder = size;
	}

	leftBorder = leftBorder + tickSize + 2*tickOffset;
	border = fm.getHeight() + fm.getLeading();


	// Set the width and height of the function display area.

	theWidth = realWidth - leftBorder - border;
	theHeight = realHeight - 2*border;
				       

	// Determine scaling factors.

	sFactorX = (theWidth-1) / (theFunction.higherX - theFunction.lowerX);
	sFactorY = (theHeight-1) / (theFunction.higherY - theFunction.lowerY);

    }

	
    private synchronized void resetLimits () {

	// Set the limits of integration
	
	lowerLimit = theFunction.lowX;
	upperLimit = theFunction.highX;

    }



    //--------------------------------------------------
    // Update the display while a limit line is being dragged
    //--------------------------------------------------

    public synchronized void update (Graphics g) {

	// How far has the mouse moved since the last update?

	double deltaX  = (double)(mouseX-grabX) / 
	                  theWidth * (theFunction.higherX-theFunction.lowerX);
	grabX = mouseX;


	// Change the limits proportionately, always rounding off to
	// hundreths.

	if (grabbedLower) {
	    double oldLowerLimit = lowerLimit;
	    lowerLimit += deltaX;
	    if (lowerLimit < theFunction.lowX)
		lowerLimit = theFunction.lowX;
	    if (lowerLimit > upperLimit) lowerLimit = upperLimit;
	    lowerLimit = Math.round(lowerLimit*100) / 100.0;
	    if (oldLowerLimit == lowerLimit) return;
	}

	else if (grabbedUpper) {
	    double oldUpperLimit = upperLimit;
	    upperLimit += deltaX;
	    if (upperLimit > theFunction.highX)
		upperLimit = theFunction.highX;
	    if (upperLimit < lowerLimit) upperLimit = lowerLimit;
	    upperLimit = Math.round(upperLimit*100) / 100.0;
	    if (oldUpperLimit == upperLimit) return;
	}


	// Paint everything all over again.

	paint(g);

    }
	
    

    //--------------------------------------------------
    // Paint the graph of the function, including axes.
    //--------------------------------------------------

    private void paintFunction (Graphics g) {
    
	// Draw the axes
	
	g.setColor(axesColor);
	g.drawLine(transX(0), transY(theFunction.lowerY),
		   transX(0), transY(theFunction.higherY));
	g.drawLine(transX(theFunction.lowerX), transY(0),
		   transX(theFunction.higherX), transY(0));

	
	// Draw the function

	g.setColor (plotColor);
	double val = theFunction.lowerX;
	int limit = theWidth/2;
	double step = (theFunction.higherX - theFunction.lowerX)/(double)limit;
        double valn = val + step;
        double ft = theFunction.integrand(val), fn;
        
	for (int i=1; i<limit; ++i, valn += step) {
            fn = theFunction.integrand(valn);
	    g.drawLine (transX(val),
			transY(ft),
			transX(valn),
			transY(fn));
            
            ft = fn;
            val = valn;
	}
    }	



    //--------------------------------------------------
    // Paint the graph of the function, including axes.
    //--------------------------------------------------

    private void paintSimpson (Graphics g, double a, double b, double fa, double fm, double fb, int Iterations) {
    
	// Draw the function

        Simpson thePoly;
        thePoly = new Simpson(a,b,fa,fm,fb);

	g.setColor (boxColor);
        double limit = theWidth/Iterations;
	double step = (b-a)/limit;
	double val = a, valn = a+step;
        double ft = fa, fn;

	for (int i=1; i<limit; ++i, valn+=step) {
            fn = thePoly.integrand(valn);
	    g.drawLine (transX(val),
			transY(ft),
			transX(valn),
			transY(fn));
            ft = fn;
            val = valn;
	}
	
    }	




    //--------------------------------------------------
    // Paint the limits of integration, including labels.
    //--------------------------------------------------

    private void paintLimits (Graphics g) {

	// Draw the limit lines

	g.setColor(limitColor);
	g.drawLine(transX(lowerLimit), 0,
		   transX(lowerLimit), theHeight);
	g.drawLine(transX(upperLimit), 0,
		   transX(upperLimit), theHeight);
	

	// Draw the limit labels

	g.setColor(labelColor);
	FontMetrics fm = getFontMetrics(g.getFont());

	String lowerLabel = Double.toString(lowerLimit);
	int lowerLength = fm.stringWidth(lowerLabel);
	g.drawString(lowerLabel,
		     transX(lowerLimit)-lowerLength/2,
		     theHeight + fm.getAscent() + fm.getLeading());

	String upperLabel = Double.toString(upperLimit);
	int upperLength = fm.stringWidth(upperLabel);
	g.drawString(upperLabel,
		     transX(upperLimit)-upperLength/2,
		     theHeight + fm.getAscent() + fm.getLeading());

    }
	


    //--------------------------------------------------
    // Paint the regions and calculate the area according to the
    // integration method in use.
    //--------------------------------------------------
    
    private void paintRegions (Graphics g) {

	double step = (upperLimit-lowerLimit) / numIterations, stepo2, gauss;
	double val=lowerLimit, valn = lowerLimit+step;
	double fVal, fVal1, fVal2, fVal_mid, vleft, vright;
	int i=1;


	// Set up

	area = 0;
	g.setColor(boxColor);


	// Rectangular method
	
	if (theMethod == Function.RECTANGULAR) {
	    for (; i<=numIterations; ++i, valn+=step) {
		fVal = theFunction.integrand(valn);
		area += fVal;
		g.drawLine(transX(val), transY(0),
			   transX(val), transY(fVal));
		g.drawLine(transX(val), transY(fVal),
			   transX(valn), transY(fVal));
		g.drawLine(transX(valn), transY(fVal),
			   transX(valn), transY(0));
                val = valn;
	    }
	    area = area*step;
	}


	// Midpoint method

	else if (theMethod == Function.MIDPOINT) {
	    for (; i<=numIterations; ++i, valn+=step) {
		fVal = theFunction.integrand(val+step/2);
		area += fVal;
		g.drawLine(transX(val), transY(0),
			   transX(val), transY(fVal));
		g.drawLine(transX(val), transY(fVal),
			   transX(valn), transY(fVal));
		g.drawLine(transX(valn), transY(fVal),
			   transX(valn), transY(0));
                val = valn;
	    }
	    area = area*step;
	}		
	

	// Trapezoidal method

	else if (theMethod == Function.TRAPEZOIDAL) {
            fVal2 = theFunction.integrand(val);
            // one itration less than others for identical 
            // number of evaluations
	step = (upperLimit-lowerLimit) / (numIterations-1);
	valn = lowerLimit+step;
	    for (; i<numIterations; ++i, valn+=step) {
		fVal1 = fVal2;
		fVal2 = theFunction.integrand(valn);
		area += (fVal1 + fVal2) / 2;
		g.drawLine(transX(val), transY(0),
			   transX(val), transY(fVal1));
		g.drawLine(transX(val), transY(fVal1),
			   transX(valn), transY(fVal2));
		g.drawLine(transX(valn), transY(fVal2),
			   transX(valn), transY(0));
                val = valn;
	    }
	    area = area*step;
	}		
	    
	// Simpson method

	else if (theMethod == Function.SIMPSON) {
            int Simpson_numIterations = (numIterations)/2;
            step = (upperLimit-lowerLimit) / Simpson_numIterations;
            fVal2 = theFunction.integrand(val);
            valn = lowerLimit+step;
	    for (; i<=Simpson_numIterations; ++i, valn+=step) {
		fVal1 = fVal2;
		fVal_mid = theFunction.integrand(val+step/2);
		fVal2 = theFunction.integrand(valn);
		area += (fVal1+4*fVal_mid+fVal2)/6.0;

		g.drawLine(transX(val), transY(0),
			   transX(val), transY(fVal1));
                paintSimpson(g,val,valn,fVal1,fVal_mid,fVal2, Simpson_numIterations);
		g.drawLine(transX(valn), transY(0),
			   transX(valn), transY(fVal2));
                val = valn;
	    }
	    area = area*step;
	}		
	
	// Gauss Legendre method
	else if (theMethod == Function.GAUSSLEGENDRE) {
            int Gauss_numIterations = (numIterations)/2;
            step = (upperLimit-lowerLimit) / Gauss_numIterations;
            val = lowerLimit;
            valn = lowerLimit+step;
            stepo2 = step/2.0;
            gauss = oneoversqrtthree*stepo2;
            
	    for (; i<=Gauss_numIterations; ++i, valn+=step) {
                            vleft = val+stepo2;
                            vright = vleft+gauss;
                            vleft  = vleft-gauss;
		fVal1 = theFunction.integrand( vleft ) ;
		fVal2 = theFunction.integrand(vright);
		area += (fVal1+fVal2)/2.0 ;

		g.drawLine(transX(vleft), transY(0),
			   transX(vleft), transY(fVal1));
		g.drawLine(transX(vright), transY(0),
			   transX(vright), transY(fVal2));
                double alpha, beta;
                alpha = (fVal2-fVal1)/(vright-vleft) ;
                beta = (fVal1*vright-fVal2*vleft)/(vright-vleft);

                g.drawLine(transX(val), transY( alpha*val +beta ),
                           transX(valn), transY( alpha*valn +beta ) );
                val = valn;
	    }
	    area = area*step;
	}

    }



    //--------------------------------------------------
    // Create an offscreen image of the function, title, and
    // labels, so long as one doesn't already exist.
    //--------------------------------------------------

    private void makeFunctionImage () {

	// If we already have an image, use it
	
	if (functionImage != null) return;


	// Create an image containing white borders and a filled
	// background where the function plot goes.

	functionImage = createImage(realWidth, realHeight);
	Graphics fg = functionImage.getGraphics();
	fg.setFont(getFont());
	fg.setColor(getBackground());
	fg.fillRect(0, 0, realWidth, realHeight);
	fg.setColor(backColor);
	fg.fillRect(leftBorder, border, theWidth, theHeight);


	// Put the title at the top of the plot

	fg.setColor(labelColor);

	String label = "Approximating " +
	               theFunction.name +
	               " using " +
	               Function.methodNames[theMethod].toLowerCase() +
	               " method";
	if (theMethod == Function.NONE) label = theFunction.name;

	FontMetrics fm = getFontMetrics(fg.getFont());
	int length = fm.stringWidth(label);
	fg.drawString(label, (realWidth-length)/2,
		      fm.getAscent() + fm.getLeading());


	// Translate the graphics context so that the upper and left borders
	// need no longer be taken into account.

	fg.translate(leftBorder, border);


	// Draw the tick marks and the labels.

	for (int i = 0; i < theFunction.labels.length; ++i) {
	    double tick = theFunction.labels[i];
	    fg.drawLine(-tickSize, transY(tick), -1, transY(tick));
	    fg.drawString(Double.toString(tick),
			  -tickSize - tickOffset -
 			              fm.stringWidth(Double.toString(tick)),
			  transY(tick) + fm.getAscent()/2);
	}


        // Clip to the function display area and paint the function
        // and axes.

	fg.clipRect(0, 0, theWidth, theHeight);
	paintFunction(fg);

    }


    //----------------------------------------------------
    // Paint the plot and calculate the area
    //----------------------------------------------------

    public synchronized void paint (Graphics g) {


	// Create a new offscreen function image if necessary.  This way,
	// we only have to create a new function image when the function
	// changes.

	makeFunctionImage();


	// We do double buffering.  Create an offscreen image for the
	// the entire display.

	Image window = createImage(realWidth, realHeight);
	Graphics g1 = window.getGraphics();
	g1.setFont(getFont());


	// Put the function image on the offscreen image, then translate
	// and add the limits and regions.

	g1.drawImage(functionImage, 0, 0, backColor, null);
	g1.translate(leftBorder, border);
	paintLimits(g1);
	paintRegions(g1);


	// Draw the image onto the display

	g.drawImage(window, 0, 0, backColor, null);


	// Update the integral display.

	idisplay.doUpdate(theFunction.integral(upperLimit) -
			  theFunction.integral(lowerLimit),
			  area);

    }
    
    
    //----------------------------------------------------
    // X-coordinate scaling
    //----------------------------------------------------
    
    public int transX (double x) {
	return (int)(Math.round(sFactorX * (x-theFunction.lowerX)));
    }

    
    //----------------------------------------------------
    // Y-coordinate scaling
    //----------------------------------------------------

    public int transY (double y) {
	return (theHeight-1) - (int)(Math.round(sFactorY * (y-theFunction.lowerY)));
    }

    
}



