/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package hamlet;

import java.applet.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.lang.*;
import hamlet.*;

/**
  * Creates an applet that displays a button on the screen.  When that
  * button is pushed, it brings up an <tt>AppFrame</tt> of the class
  * specified by
  * the <tt>NAME</tt> parameter to the applet.  For example
  * <pre>
  *    &ltapplet CODE="AppButton.class" NAME="BlockStacker.BlockStacker"&gt
  *    &ltparam name=LABEL value="Button Label"&gt
  *    &lt/applet&gt
  * </pre>
  * creates an applet that can bring up a <tt>BlockStacker.BlockStacker</tt>
  * frame.  If there is a <tt>LABEL</t> parameter, it is used to label
  * the button.
  * <P>
  * The class maintains a static registry of frames that have
  * been previously displayed, each keyed by its class name.  Static
  * functions are provided with which a frame can be removed from the
  * registry and with which events can be sent to a frame.
  * <P>
  * If, when the button is pushed, a frame of that class is already in
  * the registry, that frame is brought to the front of the display and
  * no new frame is created.
  * @author  Joseph L. Zachary (zachary@cs.utah.edu), University of Utah
  * @see AppFrame
  * @see SendButton
  */


public class AppButton extends Applet {

    private String appletName;	     // The class name of the applet
    private Button theButton;        // Button to display the frame
    private Class theClass;          // The class of the underlying frame

    /* Hash table containing frames that have been brought up.  Frames
       are keyed by their class name. */

    private static Hashtable registry = new Hashtable();


    /**
     * Obtains the <tt>NAME</tt> parameter, determines the class to which it
     * refers, and displays a button that brings up frames of that
     * class.  If there is no <tt>NAME</tt> parameter, or if no class
     * by that name can be found, the applet exits.
     */
    
    public void init () {

	// Get the applet name.  If the name doesn't exist, complain
	// and exit.

	appletName = getParameter("Name");
	if (appletName == null) {
	    System.out.println("Exiting: class name is missing");
	    return;
	}


	// If the name doesn't refer to a class, complain and exit.
	
	try {
	    theClass = Class.forName(appletName);
	}
            catch (java.lang.ClassNotFoundException s) {
		System.out.println("Exiting: " + appletName +
		                     " does not refer to a class");
                return;
	    }


	// Get the label

	String label = getParameter("LABEL");
	if (label == null) label = "Launch";

	// Display the button.

	theButton = new Button(label);
	theButton.setBackground(Color.yellow);
	add(theButton);

    }


    /**
      * When the displayed button is clicked, attempts to make, register,
      * and display a frame of the class determined by <tt>init</tt>.  If
      * such a frame has already been registered, brings it to the front
      * instead.  If the class can't be instantiated with a nullary
      * constructor, or if the class isn't an <tt>AppFrame</tt> type,
      * an error message
      * is displayed.
     * @param theEvent    Event that triggered the action
     * @param arg         Argument associated with the event
     * @return            True if event handled, false otherwise
     */

    public synchronized boolean action (Event theEvent, Object arg) {

	if (theEvent.target instanceof Button &&
	    theEvent.id == Event.ACTION_EVENT &&
	    theEvent.target.equals(theButton)) {

	    AppFrame theWindow = extract(appletName);
	    if (theWindow == null) {
		try    {
		    theWindow = (AppFrame) (theClass.newInstance());
		        }
  		        catch (java.lang.InstantiationException s) {
			    System.out.println(s);
			    return(true);
			}
 		        catch (java.lang.IllegalAccessException s) {
			    System.out.println(s);
			    return(true);
			}
  		        catch (java.lang.ClassCastException s) {
			    System.out.println(s);
			    return(true);
			}
		register(theWindow, appletName);
	    }
	    theWindow.show(true);
	    theWindow.toFront();
	    
	    return true;
	}

	return(false);

    }




    //--------------------------------------------------
    // Inserts and retrieves from the object hash table
    //--------------------------------------------------

    /* Adds obj to the hash table under its class name. */

    private synchronized static void register (Object obj, String className) {
	registry.put(className, obj);
    }

    /* Extracts the object from the hash table. */

    private synchronized static AppFrame extract (String className) {
	return((AppFrame) (registry.get(className)));
    }

    /**
     * Removes the object of the specified class from the registry,
     * and stops displaying it.
     * Does nothing if no object exists with that class name.
     * @param className  The class name of the object to be removed
     */

    public synchronized static void remove (String className) {
	AppFrame frame = (AppFrame) (extract(className));
	if (frame != null) {
	    registry.remove(className);
	    frame.show(false);
	    frame.dispose();
	}
    }

    /**
     * Calls the handleMessage method of the <tt>AppFrame</tt> of the
     * specified class from the registry.  Does nothing if no object
     * exists with that class name.
     * @param className   The class name of the object to be removed
     * @param msg   The argument to be sent along with the event
     */

    public synchronized static void sendMessage (String className,
						 String msg) {
	AppFrame obj = extract(className);
	if (obj == null) return;
	obj.handleMessage(msg);
    }


}
