/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package hamlet;

import java.awt.*;


/**
  * This class should be the superclass of all applications intended
  * to be displayed via the <tt>AppButton</tt> applet and messaged
  * via the <tt>SendButton</tt> applet.
  * An <tt>AppFrame</tt> provides a frame with a menu bar in
  * which the application can
  * run.  The menu bar contains <tt>File</tt>, <tt>Font</tt>, and
  * <tt>Help</tt> menus.
  * <P>
  * The <tt>File</tt> menu provides three options: <tt>Reset</tt>,
  * <tt>Refresh</tt>,
  * <tt>Hide</tt>, and <tt>Quit</tt>.  <tt>Reset</tt> invokes the
  * <tt>reset</tt> method
  * (which by default does nothing), <tt>Refresh</tt> refreshes the
  * display of the applet, and <tt>Hide</tt> removes the frame from
  * the display, and <tt>Quit</tt> removes the frame from the display and
  * from the <tt>AppButton</tt> registry.
  * <P>
  * The <tt>Help</tt> menu provides one option, <tt>With Applet</tt>,
  * which provides help for the application.
  * <P>
  * <tt>AppFrame</tt> objects provide one additional service.  Messages
  * that are sent via <tt>SendButton</tt> are intercepted and passed
  * to the <tt>handleMessage</tt> method, which by default does nothing.
  * <P>
  * Every subclass needs to customize several variables and methods, as
  * documented below.
  * @see AppButton
  * @see SendButton
  */

public class AppFrame extends Frame {


    /** Appropriate help text should be assigned to this variable */

    protected String helpText = "No help is available for this applet.";


    /** Additional entries can be added to <tt>menuBar</tt> via MenuBar.add */

    protected MenuBar menuBar;


    // Four buttons in the file menu.

    private MenuItem resetButton;
    private MenuItem refreshButton;
    private MenuItem hideButton;
    private MenuItem quitButton;


    // Two buttons in the font menu.

    private MenuItem smallerButton;
    private MenuItem largerButton;

    /** Additional entries can be added to <tt>helpMenu</tt> via Menu.add */

    protected Menu helpMenu;


    // The button in the help menu and the dialog it spawns.

    private MenuItem helpButton;
    private HelpDialog helpDialog;


    // The coordinates of the frame on the screen.  This is used to
    // try to work around a bug in the way that window resizing is
    // handled.

    private int frameX;
    private int frameY;


    /**
      * Sets up the default menus.  The constructor for the
      * subclass can add custom menus to the menu bar.
      */

    public AppFrame () {

	menuBar = new MenuBar();

	Menu fileMenu = new Menu("File");
	resetButton = new MenuItem("Reset");
	refreshButton = new MenuItem("Refresh");
	hideButton = new MenuItem("Hide");
	quitButton = new MenuItem("Quit");
	fileMenu.add(resetButton);
	fileMenu.add(refreshButton);
	fileMenu.add(hideButton);
	fileMenu.add(quitButton);
	menuBar.add(fileMenu);

	Menu fontMenu = new Menu("Font");
	largerButton = new MenuItem("Larger");
	smallerButton = new MenuItem("Smaller");
	fontMenu.add(largerButton);
	fontMenu.add(smallerButton);
//	menuBar.add(fontMenu);

	helpMenu = new Menu("Help");
	helpButton = new MenuItem("With Applet");
	helpMenu.add(helpButton);
	menuBar.add(helpMenu);
	menuBar.setHelpMenu(helpMenu);

	setMenuBar(menuBar);
	pack();

    }

 

    /** Resets the application to its initial state.  This is called
      * when the <tt>Reset</tt> menu item is selected.  It should be
      * overridden to do something reasonable.
      */

    public void reset () {
    }


    /** This is called whenever a <tt>SendButton</tt> message is
      * sent.  It should be overriden to do something reasonable.
      * @param msg   The message from the <tt>SendButton</tt>
      */

    protected void handleMessage (String msg) {
    }


    /** Handles the four default menu entries.  If this is overridden
      * to handle other menu entries or controls, the first statement
      * should be
      * <pre>
      *   if (super.action(theEvent, arg)) return(true);
      * </pre>
      * @param theEvent    Event that triggered the action
      * @param arg         Argument associated with the event
      * @return            True if event handled, false otherwise
      */
   
    public boolean action (Event theEvent, Object arg) {

	if (super.action(theEvent, arg)) return(true);

	if (theEvent.target instanceof MenuItem) {
 
            // Only do ACTION_EVENT events
            
            if (theEvent.id != Event.ACTION_EVENT)
		return false;

	    if (theEvent.target.equals(helpButton)) {
		if (helpDialog == null) {
		    helpDialog = new HelpDialog("Help for " + getTitle());
		    helpDialog.setText(helpText);
		}
		helpDialog.show(true);
		return true;
	    }

	    if (theEvent.target.equals (refreshButton)) {
		show(false);
		show(true);
	    }

	    if (theEvent.target.equals (hideButton)) {
		hide();
		return true;
	    }

	    if (theEvent.target.equals (resetButton)) {
		reset();
		return true;
	    }

	    if (theEvent.target.equals (quitButton)) {
		AppButton.remove(getClass().getName());
		return(true);
	    }

	    if (theEvent.target.equals (smallerButton)) {
		setFont(new Font("Helvetica", Font.PLAIN, 10));
		show(false);
		show(true);
	    }

	    if (theEvent.target.equals (largerButton)) {
		setFont(new Font("Helvetica", Font.PLAIN, 20));
		show(false);
		show(true);
	    }

	}


	// Unhandled events
	
	return false;
    } 


    /** Dispatches events.  We have overridden it here so that we keep
      * track of the approximate coordinates of the upper left corner
      * of the frame, so that we can mask a bug in reshape.
      * @param evt  The event to handle
      */

    public boolean handleEvent (Event evt) {
	if (evt.id == Event.WINDOW_MOVED) {
	    if (evt.x != 0 && evt.y != 0) {
		frameX = evt.x;
		frameY = evt.y;
	    }
	}
	return(super.handleEvent(evt));
    }


    /** Reshapes the frame.  There is a bug that sometimes causes
      * the frame to move to the upper left hand corner of the
      * display.  We try to detect that here and fix it by
      * substituting in the last known coordinates of the corner.
      * Unfortunately, this doesn't work perfectly.
      @param x  x-coordinate
      @param y  y-coordinate
      @param w  width
      @param h  height
      */

    public void reshape (int x, int y, int w, int h) {
	if (x == 0 && y == 0) {
	    x = frameX;
	    y = frameY;
	}
	super.reshape(x, y, w, h);
    }

}
