/****************************************************************************

   Copyright (C) 1995 University of Utah

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the Free
   Software Foundation, 675 Massachusetts Avenue, Cambridge, MA 02139 USA;
   either version 1, or (at your option) any later version.

   The University of Utah disclaims all warranties with regard to this
   software, including all implied warranties of merchantability and fitness.
   In no event shall the University of Utah be liable for any special, indirect
   or consequential damages or any damages whatsoever resulting from loss of
   use, data or profits, whether in an action of contract, negligence or other
   tortious action, arising out of or in connection with the use or performance
   of this software.

*****************************************************************************/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xmu/WinUtil.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>

#include <ctype.h>
#include <stdlib.h>
#include <string.h>

#include "comm.h"
#include "control.h"
#include "server.h"
#include "xdialog.h"


/* Sets the BAD_WINDOW flag if it notices a bad window error.  It ignores
   all other kinds of errors. */

static int BAD_WINDOW = 0;

static int HamletErrorHandler (Display *d, XErrorEvent *error) {
    if (error->error_code == BadWindow) BAD_WINDOW = 1;
    return(0);
}



/* Initializes by opening the X display, determining the screen, and setting up
   the error handler.  If the screen can't be opened, we complain and exit. */

Communicator::Communicator () {

    display = XOpenDisplay(getenv("DISPLAY"));

    if (display == 0) {
	fprintf(stderr, "Hamlet unable to access X display\n");
	exit(1);
    }

    screen = DefaultScreen(display);

    XSetErrorHandler(&HamletErrorHandler);

}



/* Close the display. */

Communicator::~Communicator () {
    if (display) XCloseDisplay(display);
}




/* The array "masks" maps strings to X11 event masks.  Two strings, one an
   abbreviation and one a full name, map to each event mask. */

struct maskmap {
    char *name;
    long mask;
};

static maskmap masks[] = {
    {"NE", NoEventMask},
    {"KP", KeyPressMask},
    {"KR", KeyReleaseMask},
    {"BP", ButtonPressMask},
    {"BR", ButtonReleaseMask},
    {"EW", EnterWindowMask},	
    {"LW", LeaveWindowMask},	
    {"PM", PointerMotionMask},	
    {"PMH", PointerMotionHintMask},	
    {"B1M", Button1MotionMask},	
    {"B2M", Button2MotionMask},	
    {"B3M", Button3MotionMask},	
    {"B4M", Button4MotionMask},	
    {"B5M", Button5MotionMask},	
    {"BM", ButtonMotionMask},	
    {"KS", KeymapStateMask},	
    {"E", ExposureMask},		
    {"VC", VisibilityChangeMask},	
    {"SN", StructureNotifyMask},
    {"RR", ResizeRedirectMask},
    {"SsN", SubstructureNotifyMask},
    {"SsR", SubstructureRedirectMask},
    {"FC", FocusChangeMask},
    {"PC", PropertyChangeMask},
    {"CC", ColormapChangeMask},
    {"OGB", OwnerGrabButtonMask},
    {"NoEvent", NoEventMask},
    {"KeyPress", KeyPressMask},
    {"KeyRelease", KeyReleaseMask},
    {"ButtonPress", ButtonPressMask},
    {"ButtonRelease", ButtonReleaseMask},
    {"EnterWindow", EnterWindowMask},	
    {"LeaveWindow", LeaveWindowMask},	
    {"PointerMotion", PointerMotionMask},	
    {"PointerMotionHint", PointerMotionHintMask},	
    {"Button1Motion", Button1MotionMask},	
    {"Button2Motion", Button2MotionMask},	
    {"Button3Motion", Button3MotionMask},	
    {"Button4Motion", Button4MotionMask},	
    {"Button5Motion", Button5MotionMask},	
    {"ButtonMotion", ButtonMotionMask},	
    {"KeymapState", KeymapStateMask},	
    {"Exposure", ExposureMask},		
    {"VisibilityChange", VisibilityChangeMask},	
    {"StructureNotify", StructureNotifyMask},
    {"ResizeRedirect", ResizeRedirectMask},
    {"SubstructureNotify", SubstructureNotifyMask},
    {"SubstructureRedirect", SubstructureRedirectMask},
    {"FocusChange", FocusChangeMask},
    {"PropertyChange", PropertyChangeMask},
    {"ColormapChange", ColormapChangeMask},
    {"OwnerGrabButton", OwnerGrabButtonMask},
    {NULL, 0L}
};



/* Returns the event mask corresponding to maskName.  If there is no
   correspondent event mask, notify the user and return 0. */

static long findMask (char *maskName) {
    for (int i = 0; masks[i].name; i++)
	if (strcmp(masks[i].name, maskName) == 0) return(masks[i].mask);
    notifyUser("Unknown event mask used in Hamlet directive:\n", maskName);
    return(0L);
}
  


#ifdef sun4
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2
#endif

/* Read the command from the file. */

static char *readCommand (FILE *file) {

    long current = ftell(file);
    fseek(file, 0, SEEK_END);
    long size = ftell(file) - current;
    if (fseek(file, current, SEEK_SET)) return(NULL);

    char *command = new char[size+1];
    if (!command) return(NULL);

    if (size != fread(command, 1, size, file)) {
	delete command;
	return(NULL);
      }

    command[size] = '\0';
    return(command);

}



/* Tokenizes a string in a way similar to strtok. */

static char *nextToken (char **str, char delim) {
    char *result = *str;
    if ((!result) || (!*result)) return(NULL);
    while ((**str) && (**str != delim)) (*str)++;
    if (**str) {
      **str = '\0';
      (*str)++;
    }
    return(result);
}
  


/* Divide "handle" into the name of an application class and an event mask.
   Interpert "file" to determine what is to be sent and where it is to
   be sent.  If all is false, stop when <HEAD> is encountered. */

void Communicator::send (char *handle, FILE *file, int all) {


    /* Divide the handle into name and mask.  Retain the entire
       handle, also. */

    char *handleCopy = new char[strlen(handle)+1];
    if (!handleCopy) {
       	notifyUser("Unable to send command\n",
		   "Notify hamlet@cs.utah.edu\n");
   	return;
    }
    strcpy(handleCopy, handle);

    char *name = nextToken(&handleCopy, '|');

    long mask = 0L;
    char *condition;
    while (condition = nextToken(&handleCopy, '|'))
	mask |= findMask(condition);


    /* Get control information from the ".hamletrc" file. */

    Control control;


    /* If there's a confirmed handle mapping available, go ahead
       and use it.  If it doesn't work, remove the mapping and continue as
       if one never existed. */

    Window window, subwindow;
    int confirm;


    /* Determine the length of the command and read it into a buffer. */

    char *command = readCommand(file);
    if (command == NULL) {
	notifyUser("Unable to send command\n",
		   "Notify hamlet@cs.utah.edu");
	return;
    }


    /* Truncate the command if necessary. */

    char *head = strstr(command, "\n<HEAD>\n");
    if (all==0 && head) *head = '\0';

    control.fetch(handle, window, confirm);

    if (window && confirm) {
	if (!doSendString(command, window)) {
	    delete command;
	    return;
	}
	else
	    control.remove(handle);
    }
    

    /* Ask the user what to do. */

    char *questions[] = {
	"Don't send the command",
	"Send the command, continue asking for confirmation",
	"Send the command, stop asking for confirmation",
	NULL};

    int answer = askUser(questions,
			 "Hamlet has been asked to send the following "
			 "command to a ", name, " window:\n\n",
			 command,
			 "\n\nPlease choose one of the following:");

    
    confirm = (answer == 3);

    /* If the first choice was selected, remove any entry and return. */

    if (answer == 1) {
	control.remove(handle);
	delete command;
	return;
    }

    /* If an unconfirmed mapping exists, try it out. */

    if (window) {
	if (!doSendString(command, window)) {
	    control.insert(handle, window, confirm);
	    delete command;
	    return;
	}
	else
	    control.remove(handle);
    }

	
    /* There was either no mapping or the one we had didn't work.  Get
       one from the user and try it. */

    /* Ask the user to pick a window. */

    xdialogmsg xmsg("Click the mouse in the window to which\n"
		    "the command should be sent");

    subwindow = 0;
    window = xmsg.activate();
    if (mask) subwindow = doXFindSubWindow(window, mask);

    /* If there's a subwindow, try it out. */
    
    if (subwindow && !doSendString(command, subwindow)) {
	control.insert(handle, subwindow, confirm);
	delete command;
	return;
    }

    /* The subwindow didn't work.  Try the root window instead. */

    if (window && !doSendString(command, window)) {
	control.insert(handle, window, confirm);
	delete command;
	return;
    }

    /* Everything has failed.  Notify the user and give up. */

    notifyUser("Unable to send command\n"
	       "The window that you selected did not work");

    delete command;
    return;

}




/* Looks for a window in the tree of "base" that is interested in the
   events specified in the event mask "mask". */

Window Communicator::doXFindSubWindow (Window base, long mask) {

    unsigned int nchildren;
    Window root, parent, *children;
    XWindowAttributes xwa;
    
    XGetWindowAttributes(display, base, &xwa);
    if ((xwa.all_event_masks & mask) == mask) return(base);
    
    if (!XQueryTree(display, base, &root, &parent, &children, &nchildren))
	return(None);
    
    Window winReturn = None;
    for (int i = 0; (i < nchildren) && (winReturn == None); i++) {
	winReturn = doXFindSubWindow(children[i], mask);
    }

    if (children) XFree((char *) children);
    return(winReturn);

}
    


/*
 *	Fills in the XEvent structure for a keypress event.  Does not
 *      fill in the character and mode.
 *
 *	Mode should be formed from the key masks defined in X11/X.h:
 *	  0			Treat as straight character.
 *	  ShiftMask		Shift character.
 *	  LockMask		Caps lock character.
 *	  ControlMask		Control character.
 *	  Mod1Mask		Mod1 [Meta] character.
 *	  Mod2Mask		Mod2 character.
 *	  Mod3Mask		Mod3 character.
 *	  Mod4Mask		Mod4 character.
 *	  Mod5Mask		Mod5 character.
 *	Mode can be set to a logical OR of any of the above.
 *
 */

void Communicator::buildKeypressEvent(Window w, XEvent &event) {

   event.type = KeyPress;
   event.xkey.type = KeyPress;
   event.xkey.serial = 0;
   event.xkey.send_event = True;
   event.xkey.display = display;
   event.xkey.window = w;
   event.xkey.root = RootWindow(display, screen);
   event.xkey.subwindow = w;
   event.xkey.time = CurrentTime;
   event.xkey.x = 10;
   event.xkey.y = 10;
   event.xkey.x_root = 0;
   event.xkey.y_root = 0;
   event.xkey.same_screen = False;
   
}


/* Translate a key symbol into a KeySym.  Deals with the special symbols
   "Ctrl", "Shift", and "Meta". */

static KeySym Translate (const char *keysym) {
    if (!keysym) return(NoSymbol);
    if (!strcmp(keysym, "Ctrl")) return(XK_Control_L);
    else if (!strcmp(keysym, "Meta")) return(XK_Meta_L);
    else if (!strcmp(keysym, "Shift")) return(XK_Shift_L);
    else return(XStringToKeysym(keysym));
}




/* Sends everything in the "file" up to (but not including) the new newline or
   EOF to the "window".  In the message, a backslash is treated as a quote
   character, so the backslash is not transmitted.  Anything delimited by angle
   brackets is treated as the name of a key (the string corresponding to a
   KeySym without the "XK_" part).  Formatting errors in the strings being
   transmitted as quietly ignored.  The assumption is that the user can detect
   the errors from what is transmitted. */

int Communicator::doSendString (char *msg, Window window) {

    BAD_WINDOW = 0;

    /* Save the current input focus and then set it to the window
       to which we wish to communicate. */

    Window current;
    int revert;

    XGetInputFocus(display, &current, &revert);
    XSetInputFocus(display, window, RevertToPointerRoot, CurrentTime);


    /* Initialize an X event. */

    XEvent event;
    buildKeypressEvent(window, event);

    unsigned status = 0;

    while (*msg) {

	/* If not equal to NoSymbol, we have a special symbol. */

	KeySym code = NoSymbol;


	/* Treat backslash as quote.  It had better not be the last
	   character. */

	if (*msg == '\\') {
	    msg++;
	    if (!*msg || (*msg == '\n')) break;
	}


	/* "<" and ">" delimit KeySyms.  If unrecognized, treat as
	   plain text. */

	int length = 0;

	if (*msg == '<') {
	    char *end = strchr(msg+1, '>');
	    if (end) {
		*end = '\0';
		code = Translate(msg+1);
		length = strlen(msg+1);
		*end = '>';
	    }

	}

	if (code != NoSymbol) {

	    msg = msg + length + 1;
	    switch (code) {
		
	    case XK_Shift_L:
	    case XK_Shift_R:
		status |= ShiftMask;
		break;
		
	    case XK_Control_L:
	    case XK_Control_R:
		status |= ControlMask;
		break;
		
	    case XK_Meta_L:
	    case XK_Meta_R:
		status |= Mod1Mask;
		break;
		
	    default:
		event.xkey.keycode = keysymToKeycode(code, status);
		event.xkey.state = status;
		XSendEvent(display, InputFocus, True, KeyPressMask,
			   (XEvent *) &event);
		status = 0;
		break;
		
	    }
	}


	/* Anything else is a regular ASCII character. */

	else if (*msg != '\n') {
	    event.xkey.state = status;
	    event.xkey.keycode = charToKeycode(*msg, event.xkey.state);
	    XSendEvent(display, InputFocus, True, KeyPressMask,
		       (XEvent *) &event);
	    status = 0;
	}

	msg++;

    }
  

    /* Restore the input focus. */

    XSetInputFocus(display, current, revert, CurrentTime);

    XSync(display, 0);
    return(BAD_WINDOW);

}





/* This table gives the mapping of each ASCII value to its mask and
   KeySym. */

struct conv_table_struct {
    unsigned int mode;
    KeySym keysym;
};
 

static conv_table_struct ConvTable[128] = {
    { 0, 0 },
    { ControlMask, XK_A },
    { ControlMask, XK_B },
    { ControlMask, XK_C },
    { ControlMask, XK_D },
    { ControlMask, XK_E },
    { ControlMask, XK_F },
    { ControlMask, XK_G },
    { ControlMask, XK_H },
    { ControlMask, XK_I },
    { ControlMask, XK_J },
    { ControlMask, XK_K },
    { ControlMask, XK_L },
    { ControlMask, XK_M },
    { ControlMask, XK_N },
    { ControlMask, XK_O },
    { ControlMask, XK_P },
    { ControlMask, XK_Q },
    { ControlMask, XK_R },
    { ControlMask, XK_S },
    { ControlMask, XK_T },
    { ControlMask, XK_U },
    { ControlMask, XK_V },
    { ControlMask, XK_W },
    { ControlMask, XK_X },
    { ControlMask, XK_Y },
    { ControlMask, XK_Z },
    { 0, XK_Escape },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, 0 },
    { 0, XK_space },
    { ShiftMask, XK_exclam },
    { ShiftMask, XK_quotedbl },
    { ShiftMask, XK_numbersign },
    { ShiftMask, XK_dollar },
    { ShiftMask, XK_percent },
    { ShiftMask, XK_ampersand },
    { 0, XK_apostrophe },
    { ShiftMask, XK_parenleft },
    { ShiftMask, XK_parenright },
    { ShiftMask, XK_asterisk },
    { ShiftMask, XK_plus },
    { 0, XK_comma },
    { 0, XK_minus },
    { 0, XK_period },
    { 0, XK_slash },
    { 0, XK_0 },
    { 0, XK_1 },
    { 0, XK_2 },
    { 0, XK_3 },
    { 0, XK_4 },
    { 0, XK_5 },
    { 0, XK_6 },
    { 0, XK_7 },
    { 0, XK_8 },
    { 0, XK_9 },
    { ShiftMask, XK_colon },
    { 0, XK_semicolon },
    { ShiftMask, XK_less },
    { 0, XK_equal },
    { ShiftMask, XK_greater },
    { ShiftMask, XK_question },
    { ShiftMask, XK_at },
    { ShiftMask, XK_A },
    { ShiftMask, XK_B },
    { ShiftMask, XK_C },
    { ShiftMask, XK_D },
    { ShiftMask, XK_E },
    { ShiftMask, XK_F },
    { ShiftMask, XK_G },
    { ShiftMask, XK_H },
    { ShiftMask, XK_I },
    { ShiftMask, XK_J },
    { ShiftMask, XK_K },
    { ShiftMask, XK_L },
    { ShiftMask, XK_M },
    { ShiftMask, XK_N },
    { ShiftMask, XK_O },
    { ShiftMask, XK_P },
    { ShiftMask, XK_Q },
    { ShiftMask, XK_R },
    { ShiftMask, XK_S },
    { ShiftMask, XK_T },
    { ShiftMask, XK_U },
    { ShiftMask, XK_V },
    { ShiftMask, XK_W },
    { ShiftMask, XK_X },
    { ShiftMask, XK_Y },
    { ShiftMask, XK_Z },
    { 0, XK_bracketleft },
    { 0, XK_backslash },
    { 0, XK_bracketright },
    { ShiftMask, XK_asciicircum },
    { ShiftMask, XK_underscore },
    { 0, XK_grave },
    { 0, XK_a },
    { 0, XK_b },
    { 0, XK_c },
    { 0, XK_d },
    { 0, XK_e },
    { 0, XK_f },
    { 0, XK_g },
    { 0, XK_h },
    { 0, XK_i },
    { 0, XK_j },
    { 0, XK_k },
    { 0, XK_l },
    { 0, XK_m },
    { 0, XK_n },
    { 0, XK_o },
    { 0, XK_p },
    { 0, XK_q },
    { 0, XK_r },
    { 0, XK_s },
    { 0, XK_t },
    { 0, XK_u },
    { 0, XK_v },
    { 0, XK_w },
    { 0, XK_x },
    { 0, XK_y },
    { 0, XK_z },
    { ShiftMask, XK_braceleft },
    { ShiftMask, XK_bar },
    { ShiftMask, XK_braceright },
    { ShiftMask, XK_asciitilde },
    { 0, XK_Delete }
};



/* Determines the appropriate mask and KeyCode for the Keysym "keysym". */

KeyCode Communicator::keysymToKeycode (KeySym keysym, unsigned &mask) {
    
    /* The server will tell us the proper KeyCode, but not the proper
       mask. To obtain the mask, we must try converting the KeyCode
       back to the KeySym for various index values.  We assume that
       index 0 is no mask, index 1 is ShiftMask, and anything else is
       CtrlMask. */

    KeyCode code = XKeysymToKeycode(display, keysym);
    if (code == 0) return(0);

    if (XKeycodeToKeysym(display, code, 0) == keysym);
    else if (XKeycodeToKeysym(display, code, 1) == keysym) mask |= ShiftMask;
    else mask |= ControlMask;

    return(code);
    
}


/* Determines the appropriate mask and KeyCode for the character "c". */

KeyCode Communicator::charToKeycode (char c, unsigned &mask) {
    return(keysymToKeycode(ConvTable[c].keysym, mask));
}
